# %global _disable_ld_no_undefined 1

%define docver  %{version}
%define dirver  %(echo %{version} | cut -d. -f1,2)
%define familyver %(echo %{version} | cut -d. -f1)

%define lib_major       %{dirver}
%define lib_name_orig   libpython%{familyver}
%define lib_name        %mklibname python %{lib_major}
%define develname       %mklibname python3 -d

Name:           python3
Version:        3.10.8
Release:        %mkrel 1
Summary:        An interpreted, interactive object-oriented programming language
License:        Modified CNRI Open Source License
Group:          Development/Python
URL:            https://www.python.org
Source0:        %{url}/ftp/python/%{version}/Python-%{version}.tar.xz
Source1:        %{url}/ftp/python/doc/%{docver}/python-%{docver}-docs-html.tar.bz2
#   Fedora Patches
Patch101:       00001-rpath.patch
Patch104:       00251-change-user-install-location.patch
Patch106:       00328-pyc-timestamp-invalidation-mode.patch
#   Mageia Patches
Patch201:       python3-3.5.2-skip-distutils-tests-that-fail-in-rpmbuild.patch
Patch202:       python3-3.7.1-uid-gid-overflows.patch
Patch203:       python3-3.5.2-dont-raise-from-py_compile.patch
Patch204:       python3-3.6.2-python3-config-LIBPLUSED-cmp0004-error.patch
Patch205:       0001-Fix-_statistics-module-linking.patch
#   PCLinuxOS patches
Patch301:       python3-3.10-fix-extension-linking.patch

BuildRequires:  autoconf-archive
BuildRequires:  blt
BuildRequires:  bluez-devel
BuildRequires:  bzip2-devel
BuildRequires:  db5.3-devel
BuildRequires:  expat-devel
BuildRequires:  libffi-devel
BuildRequires:  gdbm-devel
BuildRequires:  gmp-devel
BuildRequires:  ncursesw-devel
BuildRequires:  openssl-devel
BuildRequires:  python3
BuildRequires:  python3-pip
BuildRequires:  python3-setuptools
BuildRequires:  readline-devel
BuildRequires:  sqlite3-devel
BuildRequires:  tirpc-devel
BuildRequires:  termcap-devel
BuildRequires:  tcl-devel
BuildRequires:  pkgconfig(tk)
BuildRequires:  valgrind-devel
BuildRequires:  x11-server-xvfb
Requires:       %{lib_name} = %{version}-%{release}
Provides:       python(abi) = %{dirver}
Provides:       /usr/bin/python%{dirver}


%description
Python is an interpreted, interactive, object-oriented programming
language often compared to Tcl, Perl, Scheme or Java. Python includes
modules, classes, exceptions, very high level dynamic data types and
dynamic typing. Python supports interfaces to many system calls and
libraries, as well as to various windowing systems (X11, Motif, Tk,
Mac and MFC).

Programmers can write new built-in modules for Python in C or C++.
Python can be used as an extension language for applications that
need a programmable interface. This package contains most of the
standard Python modules, as well as modules for interfacing to the
Tix widget set for Tk and RPM.

Note that documentation for Python is provided in the python-docs
package.

%package -n     %{lib_name}
Summary:        Shared libraries for Python %{version}
Group:          System/Libraries

%description -n %{lib_name}
This packages contains Python shared object library.  Python is an
interpreted, interactive, object-oriented programming language often
compared to Tcl, Perl, Scheme or Java.

%package -n     %{develname}
Summary:        The libraries and header files needed for Python development
Group:          Development/Python
Requires:       %{name} = %{version}-%{release}
Requires:       %{lib_name} = %{version}-%{release}
Requires:       python-rpm-macros
Requires:       %{name}-rpm-macros
Provides:       %{name}-devel = %{version}-%{release}
Provides:       %{lib_name_orig}-devel = %{version}-%{release}

%description -n %{develname}
The Python programming language's interpreter can be extended with
dynamically loaded extensions and can be embedded in other programs.
This package contains the header files and libraries needed to do
these types of tasks.

Install %{develname} if you want to develop Python extensions.  The
python package will also need to be installed.  You'll probably also
want to install the python-docs package, which contains Python
documentation.

%package        docs
Summary:        Documentation for the Python programming language
Requires:       %{name} = %{version}-%{release}
Requires:       xdg-utils
Group:          Development/Python
BuildArch:      noarch

%description    docs
The python-docs package contains documentation on the Python
programming language and interpreter.  The documentation is provided
in ASCII text files and in LaTeX source files.

Install the python-docs package if you'd like to use the documentation
for the Python language.

%package -n     tkinter3
Summary:        A graphical user interface for the Python scripting language
Group:          Development/Python
Requires:       %{name} = %{version}
Requires:       tcl tk
Provides:       python3-tkinter = %{version}-%{release}

%description -n tkinter3
The Tkinter (Tk interface) program is an graphical user interface for
the Python scripting language.

You should install the tkinter package if you'd like to use a graphical
user interface for Python programming.

%package -n     tkinter3-apps
Summary:        Various applications written using tkinter
Group:          Development/Python
Requires:       tkinter3 = %{version}-%{release}

%description -n tkinter3-apps
Various applications written using tkinter

%prep
%setup -qn Python-%{version}
%apply_patches
find -name '*.exe' -print -delete

rm -r Modules/expat
rm configure pyconfig.h.in
sed -i 's/^AC_PREREQ/dnl AC_PREREQ/' configure.ac

mkdir html
bzcat %{SOURCE1} | tar x  -C html

find . -type f -print0 | xargs -0 perl -p -i -e 's@/usr/local/bin/python@/usr/bin/python3@'

%build
autoreconf -vfi
%configure2_5x  \
  --with-platlibdir=%{_lib} \
  --enable-ipv6 \
  --enable-loadable-sqlite-extensions \
  --enable-shared \
  --with-dbmliborder=gdbm \
  --with-system-expat \
  --with-system-ffi \
  --with-threads \
  --with-valgrind \
  --without-static-libpython \
  --without-ensurepip

export TMP="/tmp" TMPDIR="/tmp"

%make EXTRA_CFLAGS="$CFLAGS" LN="ln -sf"

%install
mkdir -p %{buildroot}%{_prefix}/lib/python%{dirver}/site-packages/__pycache__

perl -pi -e "/^CC=/ and s/distcc/gcc/" Makefile

echo '[install_scripts]' >setup.cfg
echo 'install_dir='"${RPM_BUILD_ROOT}/usr/bin" >>setup.cfg

mkdir -p %{buildroot}%{_mandir}
%makeinstall_std LN="ln -sf"

pushd %{buildroot}%{_bindir}
ln -sf python%{dirver} python%{familyver}
popd

(cd %{buildroot}%{_libdir}; ln -sf `ls libpython%{lib_major}*.so.*` libpython%{lib_major}.so)

# fix files conflicting with python2.6
mv %{buildroot}/%{_bindir}/2to3 %{buildroot}/%{_bindir}/python3-2to3

# install pynche as pynche3
cat << EOF > %{buildroot}%{_bindir}/pynche3
#!/bin/bash
exec %{_libdir}/python%{dirver}/site-packages/pynche/pynche
EOF
rm -f Tools/pynche/*.pyw
cp -r Tools/pynche %{buildroot}%{_libdir}/python%{dirver}/site-packages/

chmod 755 %{buildroot}%{_bindir}/{idle3,pynche3}

ln -f Tools/pynche/README Tools/pynche/README.pynche

install Misc/valgrind-python.supp -D %{buildroot}%{_libdir}/valgrind/valgrind-python3.supp

mkdir -p %{buildroot}%{_desktopdir}
cat > %{buildroot}%{_desktopdir}/tkinter3.desktop << EOF
[Desktop Entry]
Name=IDLE
Comment=IDE for Python3
Exec=%{_bindir}/idle3
Icon=development_environment_section
Terminal=false
Type=Application
Categories=Development;IDE;
EOF

cat > %{buildroot}%{_desktopdir}/%{name}-docs.desktop << EOF
[Desktop Entry]
Name=Python documentation
Comment=Python complete reference
Exec=%{_bindir}/xdg-open %_defaultdocdir/%{name}-docs/index.html
Icon=documentation_section
Terminal=false
Type=Application
Categories=Documentation;
EOF

# Install pathfix.py to bindir
# See https://github.com/fedora-python/python-rpm-porting/issues/24
cp -p Tools/scripts/pathfix.py %{buildroot}%{_bindir}/

# Remove shebang lines from .py files that aren't executable, and
# remove executability from .py files that don't have a shebang line:
find %{buildroot} -name \*.py \
  \( \( \! -perm /u+x,g+x,o+x -exec sed -e '/^#!/Q 0' -e 'Q 1' {} \; \
  -print -exec sed -i '1d' {} \; \) -o \( \
  -perm /u+x,g+x,o+x ! -exec grep -m 1 -q '^#!' {} \; \
  -exec chmod a-x {} \; \) \)

# Get rid of DOS batch files:
find %{buildroot} -name \*.bat -exec rm {} \;


%files
%{_bindir}/pathfix.py
%{_bindir}/pydoc%{familyver}*
%{_bindir}/python%{familyver}*
%{_bindir}/2to3-%{dirver}
%exclude %{_bindir}/python*config
%{_prefix}/lib/python%{dirver}
%{_libdir}/python%{dirver}
#%exclude %{_libdir}/python%{dirver}/ensurepip/_bundled
%exclude %{_libdir}/python%{dirver}/lib-dynload/_tkinter.*.so
%exclude %{_libdir}/python%{dirver}/site-packages/pynche
%{_libdir}/valgrind/valgrind-python3.supp
%{_mandir}/man*/*

%files -n %{lib_name}
%{_libdir}/libpython*.so.1*

%files -n %{develname}
%{_bindir}/python%{dirver}*-config
%{_bindir}/%{name}-config
%{_libdir}/libpython*.so
%{_libdir}/pkgconfig/python*.pc
%{_includedir}/python%{dirver}


%files docs
%doc html/*/*
%{_desktopdir}/%{name}-docs.desktop

%files -n tkinter3
%{_libdir}/python%{dirver}/idlelib
%{_libdir}/python%{dirver}/lib-dynload/_tkinter.*.so
%{_libdir}/python%{dirver}/site-packages/pynche
%{_libdir}/python%{dirver}/tkinter

%files -n tkinter3-apps
%{_bindir}/idle3*
%{_bindir}/pynche3
%{_desktopdir}/tkinter3.desktop

%changelog
* Tue Nov 22 2022 TerryN <terryn94 at gmail.com> 3.10.8-1pclos2022
- update

* Thu Aug 11 2022 TerryN <terryn94 at gmail.com> 3.10.6-2pclos2022
- patch CVE-2015-20107

* Wed Aug 10 2022 tex - 3.10.6-1pclos2022
 - update

* Sun Jun 19 2022 TerryN <terryn94 at gmail.com> 3.10.5-1pclos2022
- update

* Thu Apr 14 2022 TerryN <terryn94 at gmail.com> 3.10.4-1pclos2022
- update

* Wed Mar 30 2022 TerryN <terryn94 at gmail.com> 3.10.2-2pclos2022
- remove static libpython
- patch to fix linking extension modules

* Fri Mar 11 2022 TerryN <terryn94 at gmail.com> 3.10.2-1pclos2022
- update

* Sat Feb 05 2022 TerryN <terryn94 at gmail.com> 3.10.1-1pclos2022
- new version 3.10

* Tue Jan 15 2019 tex - 3.6.5-2pclos2019
- rebuild against updates

* Thu Apr 19 2018 marcin82 <fixxer1982 at gmail.com> 3.6.5-1pclos2018
- update

* Mon Nov 06 2017 bb2 <bb2> - 3.4.3-6pclos2017
- rebuild against updates

* Mon Aug 14 2017 bb2 <bb2> - 3.4.3-5pclos2017
- rebuild against updates

* Fri Jul 22 2016 bb <bb> 3.4.3-4pclos2016
- rebuild new libffi

* Sat Mar 19 2016 bb <bb> 3.4.3-3pclos2016
- split out rpm build macros

* Thu Mar 03 2016 bb <bb> 3.4.3-2pclos2016
- rebuild for new  openssl
